import os, inspect
import maya.cmds as cmds


def load_mg():
    if not cmds.pluginInfo("mg", q=True, loaded=True):
        try:
            cmds.loadPlugin("mg")
        except:
            cmds.error('Cannot load mg plugin. Please check if Myriad Grains module exists in MAYA_MODULE_PATH')
            raise
    return True


def load_mtoa():
    if not cmds.pluginInfo("mtoa", q=True, loaded=True):
        try:
            cmds.loadPlugin("mtoa")
        except:
            display_message('Cannot load mtoa plugin.', 'mtoa plugin not loaded')
            raise
    return True


def display_message(message, title="Info"):
        cmds.confirmDialog(title=title,
                                message=message,
                                button=['Close'],
                                defaultButton='Close',
                                cancelButton='Close',
                                dismissString='Close')


def get_selected_mesh_list():
    """
    :return: First selected mesh or None if no mesh selected
    """

    selection = cmds.ls(sl=True)
    if not selection:
        return None
    mesh_list = []
    for mesh in selection:
        object_type = cmds.ls(mesh, st=True)[1]
        if object_type == "transform":
            shapes = cmds.listRelatives(mesh, shapes=True)
            if not shapes:
                continue
            mesh = shapes[0]
            object_type = cmds.ls(mesh, st=True)[1]
        if object_type == "mesh":
            mesh_list.append(mesh)
    if mesh_list:
        return mesh_list
    return None


def create_automata_network():
    """
    Creates basic mgAutomata network based on selected mesh
    :return: None
    """

    load_mg()

    mesh_list = get_selected_mesh_list()
    if mesh_list is None:
        display_message('Select input mesh to create mgAutomata network template.', 'Select input mesh')
        return

    mesh = mesh_list[0]

    network_id = '_' + mesh
    automata = create_node("mgAutomata", name="mgAutomata" + network_id)
    instancer = cmds.createNode("instancer", name="instancer"+network_id)
    cube = cmds.polyCube(name="cube"+network_id, w=0.2, h=0.2, d=0.2)[0]

    dir_path = os.path.dirname(inspect.getfile(inspect.currentframe()))
    default_script_path = os.path.join(dir_path, "sampleScripts", "automata.mg" )

    cmds.setAttr(automata+".path", default_script_path, type="string")
    cmds.connectAttr(cube+".matrix", instancer+".inputHierarchy[0]")
    cmds.connectAttr(mesh+".outMesh", automata+".inMesh[0]")
    cmds.connectAttr(automata+".outPoints[0]", instancer+".inputPoints")

    cube_material = cmds.createNode("lambert", name="cubeMat"+network_id)
    cmds.setAttr(cube_material+".color", 0, 0.5, 1)
    cmds.select(cube, r=True)
    cmds.hyperShade(assign=cube_material)

    cmds.select(instancer, cube)
    group_name = cmds.group(name="mg"+network_id)
    set_channel_properties(group_name, channels=["tx", "ty", "tz", "rx", "ry", "rz", "sx", "sy", "sz"])

    mesh_transform = cmds.listRelatives(mesh, parent=True)[0]
    cmds.parent(group_name, mesh_transform)

    cmds.select(automata, r=True)


def create_sampler_network():
    """
    Creates automata network with sampler node; based on selected mesh
    :return: None
    """

    load_mg()

    mesh_list = get_selected_mesh_list()
    if mesh_list is None:
        display_message('Select input mesh to create mgSampler network template.', 'Select input mesh')
        return

    mesh = mesh_list[0]

    network_id = '_' + mesh
    dir_path = os.path.dirname(inspect.getfile(inspect.currentframe()))
    default_script_path = os.path.join(dir_path, "sampleScripts", "sampler.mg" )

    automata = create_node("mgAutomata", name="mgAutomata" + network_id)
    sampler = create_node("mgSampler", name="mgSampler" + network_id)
    mesh_to_points = create_node("mgMeshToPoints", name="mgMeshToPoints" + network_id)
    instancer = cmds.createNode("instancer", name="instancer"+network_id)
    cube = cmds.polyCube(name="cube"+network_id, w=0.2, h=0.2, d=0.2)[0]
    noise = cmds.createNode("noise", name="noise"+network_id)
    cmds.setKeyframe(noise+".time", time=0, value=0, itt="clamped", ott="clamped")
    cmds.setKeyframe(noise+".time", time=10, value=1, itt="clamped", ott="clamped")
    cmds.setInfinity(noise+".time", poi="linear")

    cmds.setAttr(automata+".path", default_script_path, type="string")
    cmds.connectAttr(cube+".matrix", instancer+".inputHierarchy[0]")
    cmds.connectAttr(mesh+".outMesh", mesh_to_points+".inMesh")
    cmds.connectAttr(mesh_to_points+".outPoints", sampler+".inPoints")
    cmds.connectAttr(sampler+".outPoints", automata+".inPoints[0]")
    cmds.connectAttr(noise+".outColor", sampler+".attr[0].color")
    cmds.connectAttr(automata+".outPoints[0]", instancer+".inputPoints")

    cmds.setAttr(sampler+".attr[0].attrName", "noise", type="string")

    cube_material = cmds.createNode("lambert", name="cubeMat"+network_id)
    cmds.setAttr(cube_material+".color", 0, 0.5, 1)
    cmds.select(cube, r=True)
    cmds.hyperShade(assign=cube_material)

    mesh_material = cmds.createNode("lambert", name="mat"+network_id)
    cmds.connectAttr(noise+".outColor", mesh_material+".color")

    cmds.select(instancer, cube)
    group_name = cmds.group(name="mg"+network_id)
    set_channel_properties(group_name, channels=["tx", "ty", "tz", "rx", "ry", "rz", "sx", "sy", "sz"])

    mesh_transform = cmds.listRelatives(mesh, parent=True)[0]
    cmds.parent(group_name, mesh_transform)

    cmds.select(automata, r=True)


def set_channel_properties(node, channels=["tx", "ty", "tz", "rx", "ry", "rz", "sx", "sy", "sz", "v"], lock=True, keyable=False):
    if node is None or cmds.objExists(node) != 1:
        cmds.warning("mg> Object does not exists: %s" % node)
        return
    for attr in channels:
        try:
            cmds.setAttr(node+'.'+attr, lock=lock, keyable=keyable)
        except Exception, e:
            cmds.error('Failed to modify channel properties: ' + str(e))


def create_node(node_type, name=None):
    load_mg()
    if name:
        node_name = cmds.createNode(node_type, name=name)
    else:
        node_name = cmds.createNode(node_type)

    if node_type == "mgAutomata":
        cmds.connectAttr("time1.outTime", node_name + ".time")

    return node_name


def create_color_attribute(node, color_name, default_value=[0.5, 0, 0]):
    attr_path = node + '.' + color_name
    cmds.addAttr(node, longName=color_name, usedAsColor=True, attributeType="float3")
    cmds.addAttr(node, longName="red_" + color_name, attributeType="float", parent=color_name)
    cmds.addAttr(node, longName="green_" + color_name, attributeType="float", parent=color_name)
    cmds.addAttr(node, longName="blue_" + color_name, attributeType="float", parent=color_name)
    cmds.setAttr(attr_path, default_value[0], default_value[1], default_value[2], type="double3")
    cmds.setAttr(attr_path, keyable=True)
    cmds.setAttr(node + ".red_" + color_name, keyable=True)
    cmds.setAttr(node + ".green_" + color_name, keyable=True)
    cmds.setAttr(node + ".blue_" + color_name, keyable=True)
    return attr_path


def create_curve(point_list, name = None):
    if not point_list:
        raise ValueError('Invalid point list; should contain sequence of triplets representing position '
                         'of subsequent curve points')

    point_count = len(point_list)
    knots = [0]
    knot_step = 1/float(point_count-1)
    for idx in xrange(0, point_count-1):
        knots.append(knots[-1]+knot_step )
    knots[-1] = 1
    if name:
        curve = cmds.curve(d=1, p=point_list, k=knots, name=name)
    else:
        curve = cmds.curve(d=1, p=point_list, k=knots)
    return curve


def create_icon(path_list, name = None, border = False):
    primary_curve = None
    min_x = min_y = max_x = max_y = None
    for path in path_list:
        for point in path:
            if min_x is None:
                min_x = max_x = point[0]
                min_y = max_y = point[1]
            else:
                if point[0] < min_x:
                    min_x = point[0]
                if max_x < point[0]:
                    max_x = point[0]
                if point[1] < min_y:
                    min_y = point[1]
                if max_y < point[1]:
                    max_y = point[1]

        curve = create_curve(path)
        if primary_curve is None:
            primary_curve = curve
        else:
            curve_shape = cmds.listRelatives(curve, shapes=True)[0]
            cmds.parent(curve_shape, primary_curve, s=True, r=True)
            cmds.delete(curve)

    if border:
        margin = 0.1
        min_x -= margin
        max_x += margin
        min_y -= margin
        max_y += margin

        border = create_curve([(min_x, min_y, 0), (max_x, min_y, 0), (max_x, max_y, 0), (min_x, max_y, 0), (min_x, min_y, 0)])
        border_shape = cmds.listRelatives(border, shapes=True)[0]
        cmds.parent(border_shape, primary_curve, s=True, r=True)
        cmds.delete(border)

    if name:
        primary_curve = cmds.rename(primary_curve, name)

    return primary_curve


def reset_matrix(transform):
    try:
        cmds.setAttr(transform+".translate", 0, 0, 0)
        cmds.setAttr(transform+".rotate", 0, 0, 0)
        cmds.setAttr(transform+".scale", 1, 1, 1)
    except Exception, e:
            cmds.error('Failed to reset matrix for: %s; %s' % (transform, str(e)))
            raise

