import os
import maya.cmds as cmds
from mg.widgets.Qt import QtCore, QtGui, QtWidgets
from mg.widgets import Window, get_maya_window
from scrollslot import ScrollSlot
import mg.utils


class MgBook(Window):

    # Widget constants
    WIDGET_NAME = "mgbook"
    BOOK_VERSION = "1.0.0"

    # Window scale event constants
    COMPACT_MAX_WIDTH = 478
    COMPACT_MAX_HEIGHT = 280
    COLLAPSE_INFO_HEIGHT = 350
    SCROLL_PREVIEW_HEIGHT = 200

    def __init__(self, parent):
        self._parent = parent
        super(MgBook, self).__init__(parent=self._parent)
        ui_file_path = os.path.join(os.path.dirname(__file__), "mgbook.ui")
        self.load_ui(ui_file_path)
        self.setWindowTitle("MgBook " + MgBook.BOOK_VERSION)

        # Set default variables
        self._current_scroll = None
        self._default_preview_path = os.path.join(os.path.dirname(__file__), "img", "preview.png")
        self._collapsed = False
        self._preview = None
        self._scroll_background = None
        self._scroll_layout = None
        self._scroll_panel_height = 0
        self._column_count = 0

        # Get scroll roots from environment variable
        if "MG_SCROLL_PATH" in os.environ:
            self._scroll_root = os.path.normpath(os.environ["MG_SCROLL_PATH"]).split(';')
        else:
            self._scroll_root = []

        self._scroll_paths = []

        # Get list of all scrolls
        for root in self._scroll_root:
            try:
                scroll_dir = os.listdir(root)
            except:
                cmds.warning("Scroll path '%s' is invalid" % root)
                continue

            for instance in scroll_dir:
                scroll_path = os.path.join(root, instance)
                if os.path.isdir(scroll_path):
                    self._scroll_paths.append(scroll_path)

        # Create Scroll per module
        self._scroll_list = []
        for scroll_root in self._scroll_paths:
            scroll = ScrollSlot(self, scroll_root)
            if scroll.is_valid():
                scroll.clicked.connect(self.set_current_scroll)
                scroll.doubleClicked.connect(self.set_current_scroll_and_execute)
                scroll.mouseOver.connect(self.set_scroll_preview)
                scroll.mouseOut.connect(self.reset_scroll_preview)
                self._scroll_list.append(scroll)

        self.setup_gui()

    def reset_scroll(self):
        if self._current_scroll:
            self._current_scroll.reset_icon()
            self._current_scroll = None
        self.reset_scroll_preview()

    @QtCore.Slot()
    def reset_scroll_preview(self):
        if self._current_scroll is None:
            self.set_default_preview()
        else:
            self.set_scroll_preview(self._current_scroll)

    @QtCore.Slot()
    def set_scroll_preview(self, scroll=None):
        if not scroll:
            scroll = self.sender()
        scroll_description = scroll.get_scroll_description()
        self.window.uiScrollInfo.setText(scroll_description)

        if scroll.get_preview_count() == 0:
            self.set_default_preview()
        else:
            self._preview = QtGui.QPixmap(scroll.get_preview_path())
            self.window.uiScrollPreview.setPixmap(self._preview)

    @QtCore.Slot()
    def set_current_scroll_and_execute(self):
        scroll = self.sender()
        self.set_current_scroll(scroll)
        self.execute_scroll()

    @QtCore.Slot()
    def set_current_scroll(self, scroll=None):
        if not scroll:
            scroll = self.sender()
        if self._current_scroll and scroll != self._current_scroll:
            self._current_scroll.reset_icon()
        self._current_scroll = scroll
        self.set_scroll_preview(self._current_scroll)
        if self._collapsed:
            self.execute_scroll()

    def setup_gui(self):
        self._scroll_background = QtWidgets.QWidget()
        size_policy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        self._scroll_background.setSizePolicy(size_policy)
        self.window.uiScroll.setBackgroundRole(QtGui.QPalette.Dark)
        self.window.uiScroll.setWidget(self._scroll_background)
        self._scroll_layout = QtWidgets.QGridLayout()
        self._scroll_layout.setAlignment(QtCore.Qt.AlignTop)
        self._scroll_background.setLayout(self._scroll_layout)
        self.resize(590, 500)
        self.resizeEvt.connect(self.update_scrolls_width)
        self._column_count = int(self.window.uiScroll.width() / 53)

        row = 0
        column = 0
        for scroll in self._scroll_list:
            scroll_icon = scroll.get_icon()
            if column == 2:
                column = 0
                row += 1
                self._scroll_panel_height += 60
            self._scroll_layout.addWidget(scroll_icon, row, column)
            column += 1

        self.set_default_preview()
        self.window.uiScrollPrevBck.setStyleSheet("background-color:black;")
        self.window.uiExecute.clicked.connect(self.execute_scroll)

    def set_default_preview(self):
        self._preview = QtGui.QPixmap(self._default_preview_path)
        self.window.uiScrollPreview.setPixmap(self._preview)
        msg = "<div style='color: LavenderBlush; font-size: 16px'>%s</div><br><br>" % "Pick scroll from list"
        self.window.uiScrollInfo.setText(msg)

    def get_scroll_column_count(self):
        if self.window.uiScroll.width() < 170:
            return 2
        return int((self.window.uiScroll.width()-15) / 60)

    @QtCore.Slot()
    def execute_scroll(self):
        try:
            mg.utils.load_mg()
            if self._current_scroll:
                result = self._current_scroll.run_scroll()
            else:
                mg.utils.display_message("Select Scroll from list", title="Info")
                return
        except ValueError, e:
            msg = "<br><span style='margin-left: 20px; font-size: 16px; color: LavenderBlush'>" + str(e) + "</span><br>"
            mg.utils.display_message(msg, "Error")
            self._current_scroll.highlight_icon(QtGui.QColor(255, 0, 0))
            return
        except Exception as e:
            import traceback
            msg = "<div style='color: LavenderBlush; font-size: 16px'>Error occured during scroll execution:</div><br>"
            msg += "<div style='margin-left: 20px; font-size: 14px; color: White'>" + str(e) + "</div>"
            msg += "<br><div style='color: LavenderBlush; font-size: 14px'>Traceback:</div><br>" +\
                   "<div style='margin-left: 20px'>" + traceback.format_exc().replace('\n', '<br>') + "</div>"
            mg.utils.display_message(msg, "Error")
            self._current_scroll.highlight_icon(QtGui.QColor(255, 0, 0))
            return
        if result and isinstance(result, basestring):
            mg.utils.display_message(result, title="Scroll executed")
        else:
            mg.utils.display_message("Scroll executed", title="Info")
        self._current_scroll.highlight_icon(QtGui.QColor(0, 255, 0))

    def update_scroll_layout(self):
        if self._column_count == self.get_scroll_column_count():
            return
        self._column_count = self.get_scroll_column_count()

        new_scroll_layout = QtWidgets.QGridLayout()
        new_scroll_layout.setAlignment(QtCore.Qt.AlignTop)

        self._scroll_panel_height = 0
        row = 0
        column = 0
        for scroll in self._scroll_list:
            scroll_icon = scroll.get_icon()
            if column == self._column_count:
                column = 0
                row += 1
                self._scroll_panel_height += 60
            new_scroll_layout.addWidget(scroll_icon, row, column)
            column += 1
        QtWidgets.QWidget().setLayout(self._scroll_layout)
        self._scroll_layout = new_scroll_layout
        self._scroll_background.setLayout(self._scroll_layout)

    @QtCore.Slot()
    def update_scrolls_width(self):
        # Routine called whenever widget resized
        if self.height() < MgBook.COLLAPSE_INFO_HEIGHT:
            self.window.uiScrollInfo.setVisible(False)
        else:
            self.window.uiScrollInfo.setVisible(True)

        if self.width() < MgBook.COMPACT_MAX_WIDTH or self.height() < MgBook.COMPACT_MAX_HEIGHT:
            self.window.uiRightFrame.setVisible(False)
            self.window.uiScroll.setMaximumWidth(2000)
            self.window.uiScroll.setMaximumHeight(2000)
            self._collapsed = True
        else:
            self._collapsed = False
            self.window.uiRightFrame.setVisible(True)
            # Adjust ScrollArea width
            slider = self.window.uiScroll.verticalScrollBar()
            if slider.isVisible():
                # If vertical slider is visible then expand scroll area to avoid hiding icons
                self.window.uiScroll.setMinimumWidth(150)
                self.window.uiScroll.setMaximumWidth(150)
            else:
                self.window.uiScroll.setMinimumWidth(127)
                self.window.uiScroll.setMaximumWidth(127)
        self.update_scroll_layout()

    def show_widget(self):
        book_geometry = None
        if not self.isVisible():
            settings_obj = QtCore.QSettings(QtCore.QSettings.UserScope, "Myriad Grains", "Myriad Grains")
            book_geometry = settings_obj.value(MgBook.WIDGET_NAME)
        if book_geometry:
            self.restoreGeometry(book_geometry)
        self.show()
        self.activateWindow()
        if self.windowState() == QtCore.Qt.WindowMinimized:
            # Restore minimized window
            self.setWindowState(QtCore.Qt.WindowNoState)

    def closeEvent(self, event):
        self.save_geometry(MgBook.WIDGET_NAME)
        self.reset_scroll()


def show():
    book.show_widget()

book = MgBook(get_maya_window())
