import os
import random
from mg.widgets.Qt import QtCore, QtGui, QtWidgets


class ScrollSlot(QtCore.QObject):

    ICON_SIZE = 50

    clicked = QtCore.Signal()
    doubleClicked = QtCore.Signal()
    mouseOver = QtCore.Signal()
    mouseOut = QtCore.Signal()

    def __init__(self, parent, scroll_path):
        super(ScrollSlot, self).__init__()
        self._parent = parent
        self._scroll_path = scroll_path
        self._script_path = os.path.join(self._scroll_path, "scroll.py")
        self._preview_path = os.path.join(self._scroll_path, "preview")
        absolute_path, self._directory = os.path.split(scroll_path)

        script_globals = dict()
        script_locals = dict()

        if os.path.isfile(self._script_path):
            try:
                execfile(self._script_path, script_globals, script_locals)
            except:
                print "MG> Cannot read scroll:", self._script_path
        else:
            print "MG> Cannot find scroll:", self._script_path
            return

        self._scroll_function = script_locals.get("scroll")
        self._name = script_locals.get("name")

        if self._name is None:
            self._name = self._directory

        self._description = script_locals.get("description")

        if self._description is None:
            self._description = "No description"

        # Create icons
        icon_path = os.path.join(self._scroll_path, "scroll.png")
        if os.path.exists(icon_path):
            # Pick scroll icon if exists
            icon_image = QtGui.QImage(icon_path)
        else:
            # Otherwise set default icon from book package
            icon_image = QtGui.QImage(os.path.join(os.path.dirname(__file__), "img", "scroll.png"))

        if icon_image.height() != ScrollSlot.ICON_SIZE or icon_image.width() != ScrollSlot.ICON_SIZE:
            icon_image = icon_image.scaled(ScrollSlot.ICON_SIZE, ScrollSlot.ICON_SIZE)

        icon = QtGui.QPixmap.fromImage(icon_image)

        # Create ScrollIcon to display in MgBook widget
        self._icon_label = ScrollIcon(self._parent, icon)
        self._icon_label.setToolTip(self._name)

        # Catch clicked event on label and pass it to MgBook widget to set current selection state
        self._icon_label.clicked.connect(self.emit_clicked_signal)
        self._icon_label.doubleClicked.connect(self.emit_double_clicked_signal)
        self._icon_label.mouseOver.connect(self.emit_mouse_over_signal)
        self._icon_label.mouseOut.connect(self.emit_mouse_out_signal)

        # Get scroll preview
        self._preview_list = []
        if os.path.isdir(self._preview_path):
            dir_content = os.listdir(self._preview_path)
        else:
            dir_content = []
        for item in dir_content:
            filename, file_extension = os.path.splitext(item)
            if file_extension == ".png":
                self._preview_list.append(os.path.join(self._preview_path, item))

    def highlight_icon(self, color):
        self._icon_label.animate_glow_color(color)

    def reset_icon(self):
        self._icon_label.reset_icon()

    def emit_clicked_signal(self):
        self.clicked.emit()

    def emit_double_clicked_signal(self):
        self.doubleClicked.emit()

    def emit_mouse_over_signal(self):
        self.mouseOver.emit()

    def emit_mouse_out_signal(self):
        self.mouseOut.emit()

    def is_valid(self):
        return os.path.isfile(self._script_path)

    def get_label(self):
        return self._label

    def get_scroll_name(self):
        return self._name

    def get_scroll_description(self):
        description = "<div style='color: LavenderBlush; font-size: 16px'>%s</div><br><br>" % self.get_scroll_name()
        description += self._description
        return description
        
    def run_scroll(self):
        return self._scroll_function()
        
    def get_settings_window(self):
        raise NotImplementedError

    def get_icon(self):
        return self._icon_label

    def get_preview_path(self, index=0):
        return self._preview_list[index]

    def get_preview_count(self):
        return len(self._preview_list)


class ScrollIcon(QtWidgets.QLabel):

    GLOW_SIZE_ACTIVE = 20
    GLOW_SIZE_INACTIVE = 80
    GLOW_COLOR_HOVER = QtGui.QColor(255, 255, 255, 140)
    GLOW_COLOR_ACTIVE = QtGui.QColor(255, 215, 0, 160)
    GLOW_COLOR_INACTIVE = QtGui.QColor(255, 255, 255, 0)
    GLOW_SIZE_CHANGE_TIME = 350
    GLOW_COLOR_CHANGE_TIME = 5
    ICON_MOUSE_OVER_OFFSET = 5

    clicked = QtCore.Signal()
    doubleClicked = QtCore.Signal()
    mouseOver = QtCore.Signal()
    mouseOut = QtCore.Signal()

    def __init__(self, parent, icon):
        super(ScrollIcon, self).__init__(parent)
        self._parent = parent
        self._icon = icon
        self.setPixmap(self._icon)
        self._active = False
        self._effect = QtWidgets.QGraphicsDropShadowEffect(self)
        self._effect.setBlurRadius(50)
        self._effect.setOffset(0, 0)
        self._effect.setColor(ScrollIcon.GLOW_COLOR_INACTIVE)
        self.setGraphicsEffect(self._effect)
        self._glow_size_animation = QtCore.QPropertyAnimation(self._effect, "blurRadius")
        self._glow_animation_easing = QtCore.QEasingCurve(QtCore.QEasingCurve.OutBack)
        self._glow_animation_easing.setAmplitude(2.0)
        self._glow_animation_easing.setOvershoot(3.0)
        self._glow_color_animation = QtCore.QPropertyAnimation(self._effect, "color")

        self._glow_color_timer = QtCore.QTimer(self)
        self._glow_color_timer.setInterval(ScrollIcon.GLOW_COLOR_CHANGE_TIME)
        self._glow_color_timer.timeout.connect(self.blink_color)

        self._glow_size_timer = QtCore.QTimer(self)
        self._glow_size_timer.setInterval(ScrollIcon.GLOW_SIZE_CHANGE_TIME)
        self._glow_size_timer.timeout.connect(self.blink_size)

        self._prev_blur_radius = ScrollIcon.GLOW_SIZE_ACTIVE
        self._prev_color_glow = ScrollIcon.GLOW_COLOR_ACTIVE
        self._blink_color_iteration = 0
        self._blink_size_iteration = 0

        self._offset = 0
        self._offset_animation = QtCore.QPropertyAnimation(self, "offset")
        self._offset_animation.setEasingCurve(self._glow_animation_easing)

        self._display_icon = QtGui.QPixmap(ScrollSlot.ICON_SIZE, ScrollSlot.ICON_SIZE)
        self.painter = QtGui.QPainter(self._display_icon)
        self.painter.setCompositionMode(QtGui.QPainter.CompositionMode_Source)
        self.painter.setClipping(True)

    def get_offset(self):
        return self._offset

    def set_offset(self, value):
        if value < 0:
            value = -value
        self._offset = value

        self._display_icon.fill(QtGui.QColor(0, 0, 0, 0))
        self.painter.drawPixmap(self._offset, -self._offset, self._icon)
        self.painter.fillRect(0, 0, self._offset, ScrollSlot.ICON_SIZE, QtGui.QColor(0, 0, 0, 0))
        self.painter.fillRect(0, ScrollSlot.ICON_SIZE-self._offset, ScrollSlot.ICON_SIZE,
                              self._offset, QtGui.QColor(0, 0, 0, 0))
        self.setPixmap(self._display_icon)

    # Property used to animate icon offset
    offset = QtCore.Property(int, get_offset, set_offset)

    def set_glow(self, state=True):
        if state:
            self._glow_size_animation.setDuration(300)
            self._glow_size_animation.setStartValue(ScrollIcon.GLOW_SIZE_INACTIVE)
            self._glow_size_animation.setEndValue(ScrollIcon.GLOW_SIZE_ACTIVE)

            self._glow_color_animation.setDuration(150)
            self._glow_color_animation.setStartValue(ScrollIcon.GLOW_COLOR_INACTIVE)
            self._glow_color_animation.setEndValue(ScrollIcon.GLOW_COLOR_HOVER)

            self._offset_animation.setDuration(150)
            self._offset_animation.setStartValue(0)
            self._offset_animation.setEndValue(ScrollIcon.ICON_MOUSE_OVER_OFFSET)
        else:
            self._glow_size_animation.setDuration(200)
            self._glow_size_animation.setStartValue(ScrollIcon.GLOW_SIZE_ACTIVE)
            self._glow_size_animation.setEndValue(ScrollIcon.GLOW_SIZE_INACTIVE)

            self._glow_color_animation.setDuration(200)
            self._glow_color_animation.setStartValue(ScrollIcon.GLOW_COLOR_HOVER)
            self._glow_color_animation.setEndValue(ScrollIcon.GLOW_COLOR_INACTIVE)

            self._offset_animation.setDuration(450)
            self._offset_animation.setStartValue(-ScrollIcon.ICON_MOUSE_OVER_OFFSET)
            self._offset_animation.setEndValue(0)

        self._glow_size_animation.start()
        self._glow_color_animation.start()
        self._offset_animation.start()

    def enterEvent(self, event):
        if self._active:
            return
        self.set_glow(state=True)
        self.mouseOver.emit()

    def leaveEvent(self, event):
        if not self._active:
            self.set_glow(state=False)
        self.mouseOut.emit()

    def mouseDoubleClickEvent(self, event):
        self.set_active()
        self.doubleClicked.emit()

    def mousePressEvent(self, event):
        self.set_active()
        self.clicked.emit()

    def set_active(self):
        if not self._active:
            self._active = True
            self.set_glow(state=True)
            self._glow_color_animation.setDuration(50)
            self._glow_color_animation.setStartValue(ScrollIcon.GLOW_COLOR_HOVER)
            self._glow_color_animation.setEndValue(ScrollIcon.GLOW_COLOR_ACTIVE)
            self._glow_color_animation.start()
            self._glow_color_timer.start()
            self._glow_size_timer.start()

    def animate_glow_color(self, color):
        self._blink_color_iteration = -100
        self._glow_color_animation.setDuration(500)
        self._glow_color_animation.setStartValue(color)
        self._glow_color_animation.setEndValue(self._prev_color_glow)
        self._glow_color_animation.start()

    def blink_color(self):
        self._blink_color_iteration += 1
        if self._blink_color_iteration <= 1:
            return
        new_glow_color = ScrollIcon.GLOW_COLOR_ACTIVE
        new_glow_color.setAlpha(random.uniform(130, 190))
        self._glow_color_animation.setDuration(ScrollIcon.GLOW_COLOR_CHANGE_TIME)
        self._glow_color_animation.setStartValue(self._prev_color_glow)
        self._glow_color_animation.setEndValue(new_glow_color)
        self._prev_color_glow = new_glow_color
        self._glow_color_animation.start()

    def blink_size(self):
        self._blink_size_iteration += 1
        if self._blink_size_iteration <= 1:
            return
        new_glow_radius = random.uniform(15, 45)
        self._glow_size_animation.setDuration(ScrollIcon.GLOW_SIZE_CHANGE_TIME)
        self._glow_size_animation.setStartValue(self._prev_blur_radius)
        self._glow_size_animation.setEndValue(new_glow_radius)
        self._prev_blur_radius = new_glow_radius
        self._glow_size_animation.start()

    def reset_icon(self):
        if self._active:
            self._glow_color_animation.setDuration(50)
            self._glow_color_animation.setStartValue(ScrollIcon.GLOW_COLOR_ACTIVE)
            self._glow_color_animation.setEndValue(ScrollIcon.GLOW_COLOR_HOVER)
            self._glow_color_animation.start()
        self.set_glow(state=False)
        self._active = False
        self._prev_blur_radius = ScrollIcon.GLOW_SIZE_ACTIVE
        self._prev_color_glow = ScrollIcon.GLOW_COLOR_ACTIVE
        self._glow_size_timer.stop()
        self._glow_color_timer.stop()
        self._blink_color_iteration = 0
        self._blink_size_iteration = 0
