# Following variables contain information about effect defined in this file.
# This information is utilized by mgbook to display scroll description.
# However, only valid effect function definition named "scroll" is necessary to execute.

name = "Molecule"
description = "Distribute Molecules on selected mesh.<br>"
description += "<div>"
description += "Select <i>Molecule</i> curve to adjust shading properties and transformation of instances.<br>"
description += "Properties are available from Channel Box or Attribute Editor: Extra Attributes section.<br>"
description += "</div>"
tag = ["Example"]


def scroll():
    """ Apply an effect (scroll) in current scene.
    This function is executed when "Go!" button in mgbook clicked.

    returns: string to display in confirmDialog after successful execution of scroll.
    """

    import os
    import inspect
    import maya.cmds as cmds
    import mg.utils
    import mg.icons

    mg.utils.load_mtoa()

    network_id = "molecule#"
    dir_path = os.path.dirname(inspect.getfile(inspect.currentframe()))
    script_path = os.path.join( dir_path, "molecule.mg" )

    selection = cmds.ls(sl=True)

    if not selection:
        raise ValueError('Selection is empty. Select mesh to execute scroll.')

    mesh_list = mg.utils.get_selected_mesh_list()

    if not mesh_list:
        raise ValueError('No mesh selected. Select mesh to execute scroll')

    translator_missing = False

    for mesh in mesh_list:
        # Create icon, controller for effect parameters.
        control = mg.utils.create_icon(mg.icons.molecule, name="Molecule", border=True)
        cmds.setAttr(control+".overrideEnabled", 1)
        cmds.setAttr(control+".overrideColor", 22)
        mg.utils.set_channel_properties(control, lock=False)

        # Place control above mesh
        mesh_transform = cmds.listRelatives(mesh, parent=True)
        control_bounding_box = cmds.xform(control, q=True, boundingBox=True, ws=True)
        control_height = control_bounding_box[4] - control_bounding_box[1]
        mesh_bounding_box = cmds.xform(mesh_transform, q=True, boundingBox=True, ws=True)
        control_x = mesh_bounding_box[0] + (mesh_bounding_box[3] - mesh_bounding_box[0]) / 2
        control_y = mesh_bounding_box[4] + control_height + 1
        control_z = mesh_bounding_box[2] + (mesh_bounding_box[5] - mesh_bounding_box[2]) / 2
        cmds.setAttr(control + ".translate", control_x, control_y, control_z)

        automata = mg.utils.create_node("mgAutomata", name="mgAutomata_" + network_id)
        instancer = cmds.createNode("instancer", name="instancer_"+network_id)

        # Create additional control attributes available from Channel Editor
        # New attributes will drive parameters of instanced objects.

        # Transformation properties
        cmds.addAttr(control, longName="moleculeScale", attributeType="float")
        cmds.setAttr(control+".moleculeScale", keyable=True )
        cmds.connectAttr(control+".moleculeScale", automata+".inFloat[11]")
        cmds.setAttr(control+".moleculeScale", 1)

        cmds.addAttr(control, longName="scaleRandom", attributeType="float")
        cmds.setAttr(control+".scaleRandom", keyable=True )
        cmds.connectAttr(control+".scaleRandom", automata+".inFloat[6]")
        cmds.setAttr(control+".scaleRandom", 0.7)

        cmds.addAttr(control, longName="rotateMolecule", attributeType="float")
        cmds.setAttr(control+".rotateMolecule", keyable=True)
        cmds.connectAttr(control+".rotateMolecule", automata+".inFloat[5]")

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0, itt="spline", ott="spline")
        cmds.setKeyframe(animation_curve, time=20, v=1, itt="spline", ott="spline")
        cmds.connectAttr(animation_curve+".output", control+".rotateMolecule")
        cmds.setInfinity(control+".rotateMolecule", pri='linear', poi='linear')

        cmds.addAttr(control, longName="offsetAlongNormal", attributeType="float")
        cmds.setAttr(control+".offsetAlongNormal", keyable=True )
        cmds.connectAttr(control+".offsetAlongNormal", automata+".inFloat[4]")
        cmds.setAttr(control+".offsetAlongNormal", 2)

        # Shading properties
        color_a = mg.utils.create_color_attribute(control, "colorA", [0, 0.032, 0.211])
        color_b = mg.utils.create_color_attribute(control, "colorB", [0, 0.072, 0])
        cmds.connectAttr(color_a, automata+".inVector[0]")
        cmds.connectAttr(color_b, automata+".inVector[1]")

        cmds.addAttr(control, longName="randomColorA", attributeType="float")
        cmds.setAttr(control+".randomColorA", keyable=True )
        cmds.connectAttr(control+".randomColorA", automata+".inFloat[2]")

        # Add animation to attributes to change appearance of objects in time.
        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0.1, itt="auto", ott="auto")
        cmds.setKeyframe(animation_curve, time=30, v=0.2, itt="auto", ott="auto")
        cmds.connectAttr(animation_curve+".output", control+".randomColorA")
        cmds.setInfinity(control+".randomColorA", pri='oscillate', poi='oscillate')

        cmds.addAttr(control, longName="randomColorB", attributeType="float")
        cmds.setAttr(control+".randomColorB", keyable=True )
        cmds.connectAttr(control+".randomColorB", automata+".inFloat[3]")
        cmds.setAttr(control+".randomColorB", 0.3)

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0.15, itt="auto", ott="auto")
        cmds.setKeyframe(animation_curve, time=20, v=0.05, itt="auto", ott="auto")
        cmds.connectAttr(animation_curve+".output", control+".randomColorB")
        cmds.setInfinity(control+".randomColorB", pri='oscillate', poi='oscillate')

        cmds.addAttr(control, longName="offsetUV", attributeType="float")
        cmds.setAttr(control+".offsetUV", keyable=True )
        cmds.connectAttr(control+".offsetUV", automata+".inFloat[0]")

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0, itt="spline", ott="spline")
        cmds.setKeyframe(animation_curve, time=70, v=1, itt="spline", ott="spline")
        cmds.connectAttr(animation_curve+".output", control+".offsetUV")
        cmds.setInfinity(control+".offsetUV", pri='linear', poi='linear')

        cmds.addAttr(control, longName="randOffsetUV", attributeType="float")
        cmds.setAttr(control+".randOffsetUV", keyable=True )
        cmds.connectAttr(control+".randOffsetUV", automata+".inFloat[1]")
        cmds.setAttr(control+".randOffsetUV", 1)

        cmds.addAttr(control, longName="colorNoiseOffset", attributeType="float")
        cmds.setAttr(control+".colorNoiseOffset", keyable=True )
        cmds.connectAttr(control+".colorNoiseOffset", automata+".inFloat[8]")

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0, itt="spline", ott="spline")
        cmds.setKeyframe(animation_curve, time=20, v=1, itt="spline", ott="spline")
        cmds.connectAttr(animation_curve+".output", control+".colorNoiseOffset")
        cmds.setInfinity(control+".colorNoiseOffset", pri='linear', poi='linear')

        cmds.addAttr(control, longName="colorNoiseScale", attributeType="float")
        cmds.setAttr(control+".colorNoiseScale", keyable=True )
        cmds.connectAttr(control+".colorNoiseScale", automata+".inFloat[9]")
        cmds.setAttr(control+".colorNoiseScale", 1)

        cmds.addAttr(control, longName="colorShift", attributeType="float")
        cmds.setAttr(control+".colorShift", keyable=True )
        cmds.connectAttr(control+".colorShift", automata+".inFloat[10]")
        cmds.setAttr(control+".colorShift", 0)

        cmds.addAttr(control, longName="shine", attributeType="float")
        cmds.setAttr(control+".shine", keyable=True )
        cmds.connectAttr(control+".shine", automata+".inFloat[12]")

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0.3, itt="auto", ott="auto")
        cmds.setKeyframe(animation_curve, time=20, v=1, itt="auto", ott="auto")
        cmds.connectAttr(animation_curve+".output", control+".shine")
        cmds.setInfinity(control+".shine", pri='oscillate', poi='oscillate')

        # Create aiStandIn: molecule
        molecule = cmds.createNode("aiStandIn", name="molecule_SIShape_"+network_id+"#")
        molecule_transform = cmds.listRelatives(molecule, parent=True, fullPath=True)[0]
        molecule_transform = cmds.rename(molecule_transform, "molecule_SI_"+network_id+"#")
        standin_path = os.path.join( dir_path, "molecule.ass" )
        cmds.setAttr(automata+".path", script_path, type="string")
        cmds.setAttr(molecule+".dso", standin_path, type="string")
        cmds.setAttr(molecule+".standInDrawOverride", 1)
        cmds.connectAttr(molecule_transform+".matrix", instancer+".inputHierarchy[0]")
        cmds.setAttr(molecule_transform+".visibility", 0)

        # Create aiStandIn: moleculeB
        molecule_b = cmds.createNode("aiStandIn", name="molecule_SIShape_"+network_id + "#")
        molecule_b_transform = cmds.listRelatives(molecule_b, parent=True, fullPath=True)[0]
        molecule_b_transform = cmds.rename(molecule_b_transform, "molecule_SI_"+network_id + "#")
        standin_path = os.path.join( dir_path, "moleculeB.ass" )
        cmds.setAttr(molecule_b+".dso", standin_path, type="string")
        cmds.setAttr(molecule_b+".standInDrawOverride", 1)
        cmds.connectAttr(molecule_b_transform+".matrix", instancer+".inputHierarchy[1]")
        cmds.setAttr(molecule_b_transform+".visibility", 0)

        # Connect source object to automata node. Vertices will drive instances.
        cmds.connectAttr(mesh+".outMesh", automata+".inMesh[0]")
        cmds.connectAttr(automata+".outPoints[0]", instancer+".inputPoints")

        try:
            # Notice user about missing instancer node translator for current mtoa version.
            cmds.setAttr(instancer+".ai_translator", "mgAutomata", type="string")
        except:
            translator_missing = True

        cmds.setAttr(automata+".exportAttributes", "colorA colorB emitA emitB offsetV repeat", type="string")

        cmds.select(instancer, molecule_transform, molecule_b_transform)
        group_name = cmds.group(name=network_id)

        cmds.parent(group_name, mesh)
        mg.utils.reset_matrix(group_name)
        cmds.parent(group_name, world=True)
        cmds.parent(group_name, mesh)
        mg.utils.reset_matrix(group_name)
        cmds.parent(group_name, world=True)

        cmds.parentConstraint(mesh_transform, group_name, maintainOffset=True)

        cmds.select(control, r=True)

    if translator_missing:
        mtoa_version = cmds.pluginInfo("mtoa", q=True, v=True)
        error_msg = "Error: Could not set mgAutomata translator for instancer node.<br>"
        error_msg += "Missing translator for mtoa version: " + str(mtoa_version) + "<br>"
        error_msg += "Instances will render with default shading properties, ignoring per-particle values."
        raise RuntimeError(error_msg)

    return "Scroll <b>Molecule</b> executed successfully"
