# Following variables contain information about effect defined in this file.
# This information is utilized by mgbook to display scroll description.
# However, only valid effect function definition named "scroll" is necessary to execute.

name = "Spiral"
description = "Distribute Spirals on selected mesh.<br>"
description += "<div>"
description += "Select <i>Spiral</i> curve to adjust shading properties and transformation of instances.<br>"
description += "Properties are available from Channel Box or Attribute Editor: Extra Attributes section.<br>"
description += "</div>"
tag = ["Example"]


def scroll():
    import os
    import inspect
    import maya.cmds as cmds
    import mg.utils
    import mg.icons

    mg.utils.load_mtoa()

    network_id = "spiral#"
    dir_path = os.path.dirname(inspect.getfile(inspect.currentframe()))
    script_path = os.path.join( dir_path, "spiral.mg" )
    standin_path = os.path.join( dir_path, "spiral.ass" )

    selection = cmds.ls(sl=True)

    if not selection:
        raise ValueError('Selection is empty. Select mesh to execute scroll.')

    mesh_list = mg.utils.get_selected_mesh_list()

    if not mesh_list:
        raise ValueError('No mesh selected. Select mesh to execute scroll')

    translator_missing = False

    for mesh in mesh_list:
        # Create icon, controller for effect parameters.
        control = mg.utils.create_icon(mg.icons.spiral, name="Spiral", border=False)
        cmds.setAttr(control+".overrideEnabled", 1)
        cmds.setAttr(control+".overrideColor", 22)
        mg.utils.set_channel_properties(control, lock=False)

        # Place control above mesh
        mesh_transform = cmds.listRelatives(mesh, parent=True)
        control_bounding_box = cmds.xform(control, q=True, boundingBox=True, ws=True)
        control_height = control_bounding_box[4] - control_bounding_box[1]
        mesh_bounding_box = cmds.xform(mesh_transform, q=True, boundingBox=True, ws=True)
        control_x = mesh_bounding_box[0] + (mesh_bounding_box[3] - mesh_bounding_box[0]) / 2
        control_y = mesh_bounding_box[4] + control_height + 1
        control_z = mesh_bounding_box[2] + (mesh_bounding_box[5] - mesh_bounding_box[2]) / 2
        cmds.setAttr(control + ".translate", control_x, control_y, control_z)

        automata = mg.utils.create_node("mgAutomata", name="mgAutomata_" + network_id)
        instancer = cmds.createNode("instancer", name="instancer_" + network_id)

        # Create additional control attributes available from Channel Editor
        # New attributes will drive parameters of instanced objects.

        # Transformation properties
        cmds.addAttr(control, longName="offsetAlongNormal", attributeType="float")
        cmds.setAttr(control + ".offsetAlongNormal", keyable=True)
        cmds.connectAttr(control + ".offsetAlongNormal", automata + ".inFloat[4]")
        cmds.setAttr(control + ".offsetAlongNormal", 4)

        cmds.addAttr(control, longName="spiralScale", attributeType="float")
        cmds.setAttr(control + ".spiralScale", keyable=True)
        cmds.connectAttr(control + ".spiralScale", automata + ".inFloat[11]")
        cmds.setAttr(control + ".spiralScale", 1)

        cmds.addAttr(control, longName="scaleRandom", attributeType="float")
        cmds.setAttr(control + ".scaleRandom", keyable=True)
        cmds.connectAttr(control + ".scaleRandom", automata + ".inFloat[6]")
        cmds.setAttr(control + ".scaleRandom", 0.5)

        cmds.addAttr(control, longName="rotateSpiral", attributeType="float")
        cmds.setAttr(control + ".rotateSpiral", keyable=True)
        cmds.connectAttr(control + ".rotateSpiral", automata + ".inFloat[5]")

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0, itt="spline", ott="spline")
        cmds.setKeyframe(animation_curve, time=5, v=2, itt="spline", ott="spline")
        cmds.connectAttr(animation_curve + ".output", control + ".rotateSpiral")
        cmds.setInfinity(control + ".rotateSpiral", pri='linear', poi='linear')

        # Shading properties

        color_a = mg.utils.create_color_attribute(control, "colorA")
        color_b = mg.utils.create_color_attribute(control, "colorB", [0, 0, 0.5])
        cmds.connectAttr(color_a, automata+".inVector[0]")
        cmds.connectAttr(color_b, automata+".inVector[1]")

        cmds.addAttr(control, longName="randomColorA", attributeType="float")
        cmds.setAttr(control+".randomColorA", keyable=True)
        cmds.connectAttr(control+".randomColorA", automata+".inFloat[2]")
        cmds.setAttr(control + ".randomColorA", 0.2)

        cmds.addAttr(control, longName="randomColorB", attributeType="float")
        cmds.setAttr(control+".randomColorB", keyable=True )
        cmds.connectAttr(control+".randomColorB", automata+".inFloat[3]")
        cmds.setAttr(control + ".randomColorB", 0.2)

        cmds.addAttr(control, longName="offsetUV", attributeType="float")
        cmds.setAttr(control+".offsetUV", keyable=True )
        cmds.connectAttr(control+".offsetUV", automata+".inFloat[0]")

        # Add animation to attributes to change appearance of objects in time.
        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0, itt="spline", ott="spline")
        cmds.setKeyframe(animation_curve, time=70, v=1, itt="spline", ott="spline")
        cmds.connectAttr(animation_curve + ".output", control + ".offsetUV")
        cmds.setInfinity(control + ".offsetUV", pri='linear', poi='linear')

        cmds.addAttr(control, longName="randOffsetUV", attributeType="float")
        cmds.setAttr(control+".randOffsetUV", keyable=True )
        cmds.connectAttr(control+".randOffsetUV", automata+".inFloat[1]")
        cmds.setAttr(control + ".randOffsetUV", 3.14)
        
        cmds.addAttr(control, longName="colorNoise", attributeType="float")
        cmds.setAttr(control+".colorNoise", keyable=True )
        cmds.connectAttr(control+".colorNoise", automata+".inFloat[7]")
        cmds.setAttr(control + ".colorNoise", 0.5)
        
        cmds.addAttr(control, longName="colorNoiseOffset", attributeType="float")
        cmds.setAttr(control+".colorNoiseOffset", keyable=True )
        cmds.connectAttr(control+".colorNoiseOffset", automata+".inFloat[8]")

        animation_curve = cmds.createNode("animCurveTU")
        cmds.setKeyframe(animation_curve, time=0, v=0, itt="spline", ott="spline")
        cmds.setKeyframe(animation_curve, time=40, v=1, itt="spline", ott="spline")
        cmds.connectAttr(animation_curve + ".output", control + ".colorNoiseOffset")
        cmds.setInfinity(control + ".colorNoiseOffset", pri='linear', poi='linear')
        
        cmds.addAttr(control, longName="colorNoiseScale", attributeType="float")
        cmds.setAttr(control+".colorNoiseScale", keyable=True )
        cmds.connectAttr(control+".colorNoiseScale", automata+".inFloat[9]")
        cmds.setAttr(control+".colorNoiseScale", 5)

        cmds.addAttr(control, longName="colorShift", attributeType="float")
        cmds.setAttr(control + ".colorShift", keyable=True)
        cmds.connectAttr(control + ".colorShift", automata + ".inFloat[10]")
        cmds.setAttr(control + ".colorShift", 0)

        # Create aiStandIn: spiral
        spiral = cmds.createNode("aiStandIn", name="spiral_SIShape_" + network_id)
        spiral_transform = cmds.listRelatives(spiral, parent=True)[0]
        spiral_transform = cmds.rename(spiral_transform, "spiral_SI_" + network_id)
        cmds.setAttr(automata+".path", script_path, type="string")
        cmds.setAttr(spiral+".dso", standin_path, type="string")
        cmds.setAttr(spiral+ ".standInDrawOverride", 1)

        cmds.setAttr(automata+".exportAttributes", "colorA colorB emitA emitB offsetV", type="string")

        cmds.connectAttr(spiral_transform+".matrix", instancer+".inputHierarchy[0]")
        cmds.connectAttr(mesh+".outMesh", automata+".inMesh[0]")
        cmds.connectAttr(automata+".outPoints[0]", instancer+".inputPoints")
        cmds.setAttr(instancer+".ai_translator", "mgAutomata", type="string")

        cmds.setAttr(spiral_transform+".visibility", 0)

        cmds.select(instancer, spiral_transform)
        group_name = cmds.group(name=network_id)
        mesh_transform = cmds.listRelatives(mesh, parent=True)

        cmds.parent(group_name, mesh)
        mg.utils.reset_matrix(group_name)
        cmds.parent(group_name, world=True)
        cmds.parent(group_name, mesh)
        mg.utils.reset_matrix(group_name)
        cmds.parent(group_name, world=True)

        cmds.parentConstraint(mesh_transform, group_name, maintainOffset=True)

        cmds.select(automata, r=True)

    if translator_missing:
        mtoa_version = cmds.pluginInfo("mtoa", q=True, v=True)
        error_msg = "Error: Could not set mgAutomata translator for instancer node.<br>"
        error_msg += "Missing translator for mtoa version: " + str(mtoa_version) + "<br>"
        error_msg += "Instances will render with default shading properties, ignoring per-particle values."
        raise RuntimeError(error_msg)

    return "Scroll <b>Spiral</b> executed successfully"
