# Following variables contain information about effect defined in this file.
# This information is utilized by mgbook to display scroll description.
# However, only valid effect function definition named "scroll" is necessary to execute.

import os
import inspect

dir_path = os.path.dirname(inspect.getfile(inspect.currentframe()))
script_path = os.path.join(dir_path, "molecule.mg")
scroll_path = os.path.join(dir_path, "scroll.py")

name = "Template"
description = "Template Scroll.<br>"
description += "<div>"
description += "Duplicate folder '<span style='color: LavenderBlush'>%s</span>'" \
               " to create new effect.<br>" % dir_path
description += "New scroll will appear in left menu.<br>"
description += "Modify script '<span style='color: LavenderBlush'>%s</span>'" \
               " to define behaviour of new Scroll.<br>" % script_path
description += "Setup node network by altering <span style='color: LavenderBlush'>scroll()</span> function in " \
               "'<span style='color: LavenderBlush'>%s</span>'.<br><br>" % scroll_path
description += "This template perform simple input -> transformation -> output operation.<br>"
description += "Select mesh and execute to apply template effect.<br>"
description += "</div>"
tag = ["Example"]


def scroll():
    """ Apply an effect (scroll) in current scene.
    This function is executed when "Go!" button in mgbook clicked.

    returns: string to display in confirmDialog after successful execution of scroll.
    """

    import os
    import inspect
    import maya.cmds as cmds
    import mg.utils
    import mg.icons

    network_id = "template#"
    selection = cmds.ls(sl=True)

    if not selection:
        raise ValueError('Selection is empty. Select mesh to execute scroll.')

    mesh_list = mg.utils.get_selected_mesh_list()

    if not mesh_list:
        # Raise an exception to display info.
        raise ValueError('No mesh selected. Select mesh to execute scroll')

    dir_path = os.path.dirname(inspect.getfile(inspect.currentframe()))
    script_path = os.path.join(dir_path, "template.mg")

    for mesh in mesh_list:
        # Create icon, controller for effect parameters.
        control = mg.utils.create_icon(mg.icons.scroll, name="Scroll", border=False)
        cmds.setAttr(control+".overrideEnabled", 1)
        cmds.setAttr(control+".overrideColor", 22)
        mg.utils.set_channel_properties(control, lock=False)

        # Place control above mesh
        mesh_transform = cmds.listRelatives(mesh, parent=True)
        control_bounding_box = cmds.xform(control, q=True, boundingBox=True, ws=True)
        control_height = control_bounding_box[4] - control_bounding_box[1]
        mesh_bounding_box = cmds.xform(mesh_transform, q=True, boundingBox=True, ws=True)
        control_x = mesh_bounding_box[0] + (mesh_bounding_box[3] - mesh_bounding_box[0]) / 2
        control_y = mesh_bounding_box[4] + control_height + 1
        control_z = mesh_bounding_box[2] + (mesh_bounding_box[5] - mesh_bounding_box[2]) / 2
        cmds.setAttr(control + ".translate", control_x, control_y, control_z)

        automata = mg.utils.create_node("mgAutomata", name="mgAutomata_" + network_id)
        instancer = cmds.createNode("instancer", name="instancer_"+network_id)

        # Create additional control attributes available from Channel Editor.
        # New attributes will drive parameters of instanced objects.

        # Example properties
        cmds.addAttr(control, longName="offset", attributeType="float")
        cmds.setAttr(control + ".offset", keyable=True)
        cmds.connectAttr(control + ".offset", automata + ".inFloat[0]")

        cmds.addAttr(control, longName="cubeScale", attributeType="float")
        cmds.setAttr(control+".cubeScale", keyable=True )
        cmds.connectAttr(control+".cubeScale", automata+".inFloat[1]")
        cmds.setAttr(control+".cubeScale", 1)

        cmds.addAttr(control, longName="sphereScale", attributeType="float")
        cmds.setAttr(control+".sphereScale", keyable=True )
        cmds.connectAttr(control+".sphereScale", automata+".inFloat[2]")
        cmds.setAttr(control+".sphereScale", 1)

        # Create instance: cube
        cube = cmds.polyCube()[0]
        cmds.connectAttr(cube+".matrix", instancer+".inputHierarchy[0]")
        cmds.setAttr(cube+".visibility", 0)

        cube_material = cmds.createNode("lambert", name="cubeMat" + network_id)
        cmds.setAttr(cube_material + ".color", 1, 0, 0.5)
        cmds.select(cube, r=True)
        cmds.hyperShade(assign=cube_material)

        # Create instance: sphere
        sphere = cmds.polySphere(r=0.2, sx=8, sy=8)[0]
        cmds.setAttr(automata + ".path", script_path, type="string")
        cmds.connectAttr(sphere + ".matrix", instancer + ".inputHierarchy[1]")
        cmds.setAttr(sphere + ".visibility", 0)

        sphere_material = cmds.createNode("lambert", name="sphereMat" + network_id)
        cmds.setAttr(sphere_material + ".color", 0, 0.5, 1)
        cmds.select(sphere, r=True)
        cmds.hyperShade(assign=sphere_material)

        # Connect source object to automata node. Vertices will drive instances.
        cmds.connectAttr(mesh+".outMesh", automata+".inMesh[0]")
        cmds.connectAttr(automata+".outPoints[0]", instancer+".inputPoints")

        cmds.select(instancer, sphere, cube)
        group_name = cmds.group(name=network_id)

        cmds.parent(group_name, mesh)
        mg.utils.reset_matrix(group_name)
        cmds.parent(group_name, world=True)
        cmds.parent(group_name, mesh)
        mg.utils.reset_matrix(group_name)
        cmds.parent(group_name, world=True)

        cmds.parentConstraint(mesh_transform, group_name, maintainOffset=True)
        cmds.select(control, r=True)

    return "Scroll <b>Template</b> executed successfully"
